@@ -8,13 +8,17 @@
 use libfuzzer_sys::fuzz_target;
 use uu_test::uumain;
 
-use rand::seq::SliceRandom;
+use rand::prelude::IndexedRandom;
 use rand::Rng;
 use std::ffi::OsString;
 
 mod fuzz_common;
-use crate::fuzz_common::{generate_and_run_uumain, run_gnu_cmd};
+use crate::fuzz_common::CommandResult;
+use crate::fuzz_common::{
+    compare_result, generate_and_run_uumain, generate_random_string, run_gnu_cmd,
+};
 
+#[allow(clippy::upper_case_acronyms)]
 #[derive(PartialEq, Debug, Clone)]
 enum ArgType {
     STRING,
@@ -28,29 +32,6 @@ enum ArgType {
 
 static CMD_PATH: &str = "test";
 
-fn generate_random_string(max_length: usize) -> String {
-    let mut rng = rand::thread_rng();
-    let valid_utf8: Vec<char> = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
-        .chars()
-        .collect();
-    let invalid_utf8 = [0xC3, 0x28]; // Invalid UTF-8 sequence
-    let mut result = String::new();
-
-    for _ in 0..rng.gen_range(1..=max_length) {
-        if rng.gen_bool(0.9) {
-            let ch = valid_utf8.choose(&mut rng).unwrap();
-            result.push(*ch);
-        } else {
-            let ch = invalid_utf8.choose(&mut rng).unwrap();
-            if let Some(c) = char::from_u32(*ch as u32) {
-                result.push(c);
-            }
-        }
-    }
-
-    result
-}
-
 #[derive(Debug, Clone)]
 struct TestArg {
     arg: String,
@@ -58,7 +39,7 @@ struct TestArg {
 }
 
 fn generate_random_path(rng: &mut dyn rand::RngCore) -> &'static str {
-    match rng.gen_range(0..=3) {
+    match rng.random_range(0..=3) {
         0 => "/dev/null",
         1 => "/dev/random",
         2 => "/tmp",
@@ -84,6 +65,14 @@ fn generate_test_args() -> Vec<TestArg> {
             arg: "!=".to_string(),
             arg_type: ArgType::STRINGSTRING,
         },
+        TestArg {
+            arg: ">".to_string(),
+            arg_type: ArgType::STRINGSTRING,
+        },
+        TestArg {
+            arg: "<".to_string(),
+            arg_type: ArgType::STRINGSTRING,
+        },
         TestArg {
             arg: "-eq".to_string(),
             arg_type: ArgType::INTEGERINTEGER,
@@ -132,15 +121,15 @@ fn generate_test_args() -> Vec<TestArg> {
 }
 
 fn generate_test_arg() -> String {
-    let mut rng = rand::thread_rng();
+    let mut rng = rand::rng();
     let test_args = generate_test_args();
     let mut arg = String::new();
 
-    let choice = rng.gen_range(0..=5);
+    let choice = rng.random_range(0..=5);
 
     match choice {
         0 => {
-            arg.push_str(&rng.gen_range(-100..=100).to_string());
+            arg.push_str(&rng.random_range(-100..=100).to_string());
         }
         1..=3 => {
             let test_arg = test_args
@@ -149,20 +138,20 @@ fn generate_test_arg() -> String {
             if test_arg.arg_type == ArgType::INTEGER {
                 arg.push_str(&format!(
                     "{} {} {}",
-                    &rng.gen_range(-100..=100).to_string(),
+                    &rng.random_range(-100..=100).to_string(),
                     test_arg.arg,
-                    &rng.gen_range(-100..=100).to_string()
+                    &rng.random_range(-100..=100).to_string()
                 ));
             } else if test_arg.arg_type == ArgType::STRINGSTRING {
-                let random_str = generate_random_string(rng.gen_range(1..=10));
-                let random_str2 = generate_random_string(rng.gen_range(1..=10));
+                let random_str = generate_random_string(rng.random_range(1..=10));
+                let random_str2 = generate_random_string(rng.random_range(1..=10));
 
                 arg.push_str(&format!(
                     "{} {} {}",
                     &random_str, test_arg.arg, &random_str2
                 ));
             } else if test_arg.arg_type == ArgType::STRING {
-                let random_str = generate_random_string(rng.gen_range(1..=10));
+                let random_str = generate_random_string(rng.random_range(1..=10));
                 arg.push_str(&format!("{} {}", test_arg.arg, &random_str));
             } else if test_arg.arg_type == ArgType::FILEFILE {
                 let path = generate_random_path(&mut rng);
@@ -174,7 +163,7 @@ fn generate_test_arg() -> String {
             }
         }
         4 => {
-            let random_str = generate_random_string(rng.gen_range(1..=10));
+            let random_str = generate_random_string(rng.random_range(1..=10));
             arg.push_str(&random_str);
         }
         _ => {
@@ -196,39 +185,36 @@ fn generate_test_arg() -> String {
 }
 
 fuzz_target!(|_data: &[u8]| {
-    let mut rng = rand::thread_rng();
-    let max_args = rng.gen_range(1..=6);
+    let mut rng = rand::rng();
+    let max_args = rng.random_range(1..=6);
     let mut args = vec![OsString::from("test")];
 
     for _ in 0..max_args {
         args.push(OsString::from(generate_test_arg()));
     }
 
-    let (rust_output, uumain_exit_status) = generate_and_run_uumain(&args, uumain);
+    let rust_result = generate_and_run_uumain(&args, uumain, None);
 
-    // Run GNU test with the provided arguments and compare the output
-    match run_gnu_cmd(CMD_PATH, &args[1..], false) {
-        Ok((gnu_output, gnu_exit_status)) => {
-            let gnu_output = gnu_output.trim().to_owned();
-            println!("gnu_exit_status {}", gnu_exit_status);
-            println!("uumain_exit_status {}", uumain_exit_status);
-            if rust_output != gnu_output || uumain_exit_status != gnu_exit_status {
-                println!("Discrepancy detected!");
-                println!("Test: {:?}", &args[1..]);
-                println!("My output: {}", rust_output);
-                println!("GNU output: {}", gnu_output);
-                println!("My exit status: {}", uumain_exit_status);
-                println!("GNU exit status: {}", gnu_exit_status);
-                panic!();
-            } else {
-                println!(
-                    "Outputs and exit statuses matched for expression {:?}",
-                    &args[1..]
-                );
+    let gnu_result = match run_gnu_cmd(CMD_PATH, &args[1..], false, None) {
+        Ok(result) => result,
+        Err(error_result) => {
+            eprintln!("Failed to run GNU command:");
+            eprintln!("Stderr: {}", error_result.stderr);
+            eprintln!("Exit Code: {}", error_result.exit_code);
+            CommandResult {
+                stdout: String::new(),
+                stderr: error_result.stderr,
+                exit_code: error_result.exit_code,
             }
         }
-        Err(_) => {
-            println!("GNU test execution failed for expression {:?}", &args[1..]);
-        }
-    }
+    };
+
+    compare_result(
+        "test",
+        &format!("{:?}", &args[1..]),
+        None,
+        &rust_result,
+        &gnu_result,
+        false, // Set to true if you want to fail on stderr diff
+    );
 });
